# frozen_string_literal: true

#
# Cookbook:: snu_nginx_fips
# Library:: resource/snu_nginx_fips_service
#
# Copyright:: 2017, Socrata, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

require 'chef/resource'
require 'chef/resource/service'

class Chef
  class Resource
    #
    # A resource for managing the Nginx service.
    #
    class SnuNginxFipsService < Resource
      provides :snu_nginx_fips_service do |_node|
        false
      end

      default_action %i[create enable start]

      property :config_file, String, default: '/etc/nginx/nginx.conf'
      property :file_limit, Integer, default: 10_000

      #
      # The :create action needs to be defined based on the init system in use
      # and should ensure the appropriate init config is created and pointed at
      # the proper PID file, log directory, etc.
      #
      action :create do
        raise(Chef::Exceptions::UnsupportedPlatform,
              'The :create action needs to be defined for this platform!')
      end

      #
      # The :remove action needs to be defined based on the init system in use
      # and should delete the appropriate init config.
      #
      action :remove do
        raise(Chef::Exceptions::UnsupportedPlatform,
              'The :remove action needs to be defined for this platform!')
      end

      #
      # Every other standard service action should be passed on to an
      # underlying nginx service resource.
      #
      Chef::Resource::Service.allowed_actions.each do |a|
        action a do
          execute 'Validate Nginx config' do
            command "ulimit -n #{new_resource.file_limit} && " \
                    "/usr/sbin/nginx -t -c #{new_resource.config_file}"
            action :nothing
          end

          service "#{a} nginx" do
            service_name 'nginx'
            supports restart: true, reload: true, status: true
            action a
            if %i[start restart reload].include?(a)
              notifies :run, 'execute[Validate Nginx config]', :before
            end
          end
        end
      end
    end
  end
end
